==> nhder-1.0.1.tgz: package/src/downloader.js <==
/*
 * @Author: Jindai Kirin 
 * @Date: 2018-12-16 00:56:02 
 * @Last Modified by: Jindai Kirin
 * @Last Modified time: 2018-12-17 21:32:24
 */

require('colors');
const Config = require('./config');
const Fse = require('fs-extra');
const Path = require('path');
const MultiThread = require('./multi-thread');

const RETRY_MAX = 10;

class NHDownloader {
	/**
	 * Creates an instance of NHDownloader.
	 * @param {Config} conf 配置
	 * @param {*} [agent=null] 代理
	 * @memberof NHDownloader
	 */
	constructor(conf, agent = null) {
		this.config = conf;
		if (agent) this.Axios = require('axios').create({
			httpsAgent: agent
		});
		else this.Axios = require('axios');
	}

	/**
	 * Axios下载至文件
	 *
	 * @param {string} dir 下载目录
	 * @param {string} filename 文件名
	 * @param {string} url 下载地址
	 * @param {*} [axiosOption={}] Axios选项
	 * @returns 下载到文件的Promise
	 * @memberof NHDownloader
	 */
	async axiosToFile(dir, filename, url, axiosOption = {}) {
		let response;
		axiosOption.responseType = 'stream';
		await this.Axios.get(url, axiosOption).then(res => response = res);
		return new Promise((reslove, reject) => {
			response.data.pipe(Fse.createWriteStream(Path.join(dir, filename)));
			response.data.on('end', () => {
				reslove(response);
			});
			response.data.on('error', e => {
				reject(e);
			});
		});
	}

	/**
	 * 下载本子
	 *
	 * @param {*} book 本子
	 * @param {number} [current=1] 当前总进度
	 * @param {number} [total=1] 总进度
	 * @returns 多线程Promise.all
	 * @memberof NHDownloader
	 */
	download(book, current = 1, total = 1) {
		let config = this.config.getConfig();
		let tmpDirRoot = this.config.getTempDir();
		let {
			id,
			media_id,
			title,
			num_pages,
			pages
		} = book;

		console.log('\n' + String(current).green + `/${total}\t` + `[${id}] `.gray + title + ` (${num_pages} pages)`.yellow);

		let multiThread = new MultiThread(pages, config.thread);
		return multiThread.run((threadID, filename, index, total) => new Promise(async (resolve, reject) => {
			let tmpDir = Path.join(tmpDirRoot, `${id}`);
			let dlDir = Path.join(config.path, `${title} (${id})`);
			let url = `https://i.nhentai.net/galleries/${media_id}/${filename}`;
			Fse.ensureDirSync(tmpDir);
			Fse.ensureDirSync(dlDir);

			let strTemplet = `\t${String(index).green}/${total}\t${url}`;

			console.log(`  [${threadID}]${strTemplet}`);

			let success = false;
			for (let retry = 0; retry < RETRY_MAX && !success; retry++) {
				await this.axiosToFile(tmpDir, filename, url, {
					timeout: 1000 * config.timeout
				}).then(async res => {
					//文件完整性校验
					let fileSize = res.headers['content-length'];
					let dlFile = Path.join(tmpDir, filename);
					for (let i = 0; i < 15 && !Fse.existsSync(dlFile); i++)
						await sleep(200); //不明bug处理
					let dlFileSize = Fse.statSync(dlFile).size;
					if (dlFileSize == fileSize) {
						Fse.moveSync(dlFile, Path.join(dlDir, filename));
						success = true;
					} else {
						Fse.unlinkSync(dlFile);
						throw new Error('Incomplete download.');
					}
				}).catch(e => {
					success = false;
					console.error('[ERROR] '.red + e);
					if (retry + 1 < RETRY_MAX)
						console.log('  ' + `[${threadID}]`.bgYellow + strTemplet);
					else
						console.log('  ' + `[${threadID}]`.bgRed + strTemplet);
				});
			}

			success ? resolve() : reject('Max retry.');
		}));
	}
}

function sleep(ms) {
	return new Promise(resolve => {
		setTimeout(resolve, ms);
	});
}

module.exports = NHDownloader;

==> nhder-1.0.2.tgz: package/src/downloader.js <==
/*
 * @Author: Jindai Kirin 
 * @Date: 2018-12-16 00:56:02 
 * @Last Modified by: Jindai Kirin
 * @Last Modified time: 2018-12-20 02:38:00
 */

require('colors');
const Config = require('./config');
const Fse = require('fs-extra');
const Path = require('path');
const MultiThread = require('./multi-thread');

const RETRY_MAX = 10;

class NHDownloader {
	/**
	 * Creates an instance of NHDownloader.
	 * @param {Config} conf 配置
	 * @param {*} [agent=null] 代理
	 * @memberof NHDownloader
	 */
	constructor(conf, agent = null) {
		this.config = conf;
		if (agent) this.Axios = require('axios').create({
			httpsAgent: agent
		});
		else this.Axios = require('axios');
	}

	/**
	 * Axios下载至文件
	 *
	 * @param {string} dir 下载目录
	 * @param {string} filename 文件名
	 * @param {string} url 下载地址
	 * @param {*} [axiosOption={}] Axios选项
	 * @returns 下载到文件的Promise
	 * @memberof NHDownloader
	 */
	async axiosToFile(dir, filename, url, axiosOption = {}) {
		let response;
		axiosOption.responseType = 'stream';
		await this.Axios.get(url, axiosOption).then(res => response = res);
		return new Promise((reslove, reject) => {
			response.data.pipe(Fse.createWriteStream(Path.join(dir, filename)));
			response.data.on('end', () => {
				reslove(response);
			});
			response.data.on('error', e => {
				reject(e);
			});
		});
	}

	/**
	 * 下载本子
	 *
	 * @param {*} book 本子
	 * @param {number} [current=1] 当前总进度
	 * @param {number} [total=1] 总进度
	 * @returns 多线程Promise.all
	 * @memberof NHDownloader
	 */
	download(book, current = 1, total = 1) {
		let config = this.config.getConfig();
		let tmpDirRoot = this.config.getTempDir();
		let {
			id,
			media_id,
			title,
			title_dir,
			num_pages,
			pages
		} = book;

		console.log('\n' + String(current).green + `/${total}\t` + `[${id}] `.gray + title + ` (${num_pages} pages)`.yellow);

		let multiThread = new MultiThread(pages, config.thread);
		return multiThread.run((threadID, filename, index, total) => new Promise(async (resolve, reject) => {
			let tmpDir = Path.join(tmpDirRoot, `${id}`);
			let dlDir = Path.join(config.path, title_dir);
			let url = `https://i.nhentai.net/galleries/${media_id}/${filename}`;
			Fse.ensureDirSync(tmpDir);
			Fse.ensureDirSync(dlDir);

			let strTemplet = `\t${String(index).green}/${total}\t${url}`;

			console.log(`  [${threadID}]${strTemplet}`);

			let success = false;
			for (let retry = 0; retry < RETRY_MAX && !success; retry++) {
				await this.axiosToFile(tmpDir, filename, url, {
					timeout: 1000 * config.timeout
				}).then(async res => {
					//文件完整性校验
					let fileSize = res.headers['content-length'];
					let dlFile = Path.join(tmpDir, filename);
					for (let i = 0; i < 15 && !Fse.existsSync(dlFile); i++)
						await sleep(200); //不明bug处理
					let dlFileSize = Fse.statSync(dlFile).size;
					if (dlFileSize == fileSize) {
						Fse.moveSync(dlFile, Path.join(dlDir, filename));
						success = true;
					} else {
						Fse.unlinkSync(dlFile);
						throw new Error('Incomplete download.');
					}
				}).catch(e => {
					success = false;
					console.error('[ERROR] '.red + e);
					if (retry + 1 < RETRY_MAX)
						console.log('  ' + `[${threadID}]`.bgYellow + strTemplet);
					else
						console.log('  ' + `[${threadID}]`.bgRed + strTemplet);
				});
			}

			success ? resolve() : reject('Max retry.');
		}));
	}
}

function sleep(ms) {
	return new Promise(resolve => {
		setTimeout(resolve, ms);
	});
}

module.exports = NHDownloader;
